#include <string.h>
#include <malloc.h>
#include "utils.h"

static void remove_right( char *str, size_t numChars, size_t &i, size_t &o )
{
    while( (i<numChars) && (str[i+1]==' ' || str[i+1]=='\t' || str[i+1]=='\r' || str[i+1]=='\n') )
        i++;
}
static void remove_left( char *str, size_t numChars, size_t &i, size_t &o )
{
    while( (o>1) && (str[o-1]==' ' || str[o-1]=='\t'  ) ) // || str[o-1]=='\r' || str[o-1]=='\n'
        o--; 
}
static bool remove_right_of_string( const char *what, char *str, size_t numChars, size_t &i, size_t &o )
{ 
    const size_t len = strlen(what);
    for( size_t k=0; k<len; k++ ) if( str[i+k]!=what[k] ) return false;
    
    for( size_t k=0; k<len; k++ )
        str[o++] = what[k];
    i += len;
    
    return true;
}

bool minify( char * str )
{
    const size_t numChars = strlen(str);

    size_t o = 0;
    size_t i = 0;
    
    while( (i<numChars) && (str[i]==' ' || str[i]=='\t' || str[i]=='\r' || str[i]=='\n') ) i++;

    for( ; i<numChars; i++ )
    {
        // skip defines
        if( str[i]=='#' )
        {
            remove_left(str,numChars,i,o);
            str[o++]='\n';
            while( !(str[i]=='\r' || str[i]=='\n' || (str[i]=='/' && str[i+1]=='/')) )
                str[o++] = str[i++];
            continue;
        }

        // remove comments
        if( str[i]=='/' && str[i+1]=='*' ) { while( !(str[i]=='*' && str[i+1]=='/') ) i++; i++; continue; }
        if( str[i]=='/' && str[i+1]=='/' ) { remove_left(str,numChars,i,o); while( !(str[i]=='\n') ) i++; remove_right(str,numChars,i,o); continue; }

        // remove spaces
        if( str[i]=='(' || str[i]==')' || str[i]=='{' || str[i]=='}' || 
            str[i]==';' || str[i]==':' || str[i]=='+' || str[i]=='-' ||
            str[i]=='*' || str[i]=='/' || str[i]=='>' || str[i]=='<' ||
            str[i]=='|' || str[i]=='^' || str[i]=='~' ||
            str[i]==',' || str[i]=='=' || str[i]=='\r' || str[i]=='\n' )
        {
            remove_left(str,numChars,i,o);
            str[o++] = str[i];
            remove_right(str,numChars,i,o);
            continue;
        }
        else if( remove_right_of_string( "void",str,numChars,i,o) ||
                remove_right_of_string( "int",str,numChars,i,o) ||
                remove_right_of_string( "float",str,numChars,i,o) ||
                remove_right_of_string( "vec2",str,numChars,i,o) ||
                remove_right_of_string( "vec3",str,numChars,i,o) ||
                remove_right_of_string( "vec4",str,numChars,i,o) ||
                remove_right_of_string( "ivec2",str,numChars,i,o) ||
                remove_right_of_string( "ivec3",str,numChars,i,o) ||
                remove_right_of_string( "ivec4",str,numChars,i,o) ||
                remove_right_of_string( "bvec2",str,numChars,i,o) ||
                remove_right_of_string( "bvec3",str,numChars,i,o) ||
                remove_right_of_string( "bvec4",str,numChars,i,o) ||
                remove_right_of_string( "mat2",str,numChars,i,o) ||
                remove_right_of_string( "mat3",str,numChars,i,o) ||
                remove_right_of_string( "mat4",str,numChars,i,o) )
        {
            str[o++] = str[i];
            remove_right(str,numChars,i,o);
            continue;
        }

        // default, copy stuff out
        str[o++] = str[i];
    }
    str[o] = 0;
    return true;
}
