#include <stdio.h>
#include <malloc.h>
#include <string.h>

#include "utils.h"
#include "minify.h"

static bool iConvert_Bin_To_CArray( const char * srcFileName, const char * dstFileName )
{
    FILE *src = fopen( srcFileName, "rb" ); if( !src ) return false;
    FILE *dst = fopen( dstFileName, "wt" ); if( !dst ) return false;

    const char *name = utlExtractNameWithoutExtension(srcFileName);
    const size_t len = utlGetFileLength(src);
    fprintf(dst,"static const unsigned char %s[%zd] = {\n", name, len );
    for( size_t i=0; i<len; i++ )
    {
        int c = 0; fread(&c,1,1,src);
        fprintf(dst, "0x%02x%s%s", c, (i==len-1)?"};\n":", ", (((i+1)%31)==0)?"\n":"" );
    }

    fclose( src );
    fclose( dst );
    return true;
}

static bool iConvert_Text_To_CArray( FILE * file, const char * str, const char * srcFileName )
{
    char *name = utlExtractNameWithoutExtension(srcFileName);
    if( !name ) return false;

    const size_t len = strlen(str);
    fprintf( file, "static unsigned char %s[%zd] = {\n", name, len );
    for( size_t i=0; i<len; i++ )
    {
        const int c = (int)str[i];
        fprintf( file, "0x%02x%s%s", c, (i==len-1)?"};\n":", ", (((i+1)%31)==0)?"\n":"" );
    }

    free( name );

    return true;
}

static bool iConvert_Text_To_CString( FILE * file, const char * str, const char * srcFileName )
{
    char *name = utlExtractNameWithoutExtension(srcFileName);
    if( !name ) return false;

    fprintf(file, "const char *%s = \\\n", name);
    fprintf(file, "%s", str );
    fprintf(file, ";\n" );

    free( name );

    return true;
}

static bool iConvert_Text_To_Text( FILE * file, const char * str, const char * srcFileName)
{
    fprintf(file, str );
    return true;
}

//=========================================================================================

int main( int numArgs, char *args[] )
{
    if( numArgs<4 || numArgs>5 )
    {
        printf( "toArray.exe inputFile outputFile {Text_To_Text|Text_To_CString|Text_To_CArray|Binary_To_CArray} [minify]\n" );
        return -1;
    }

    const char *inputFileStr  = args[1];
    const char *outputFileStr = args[2];
    const char *typeStr       = args[3];

    bool doMinify  = false;
    for( int i=4; i<numArgs; i++ )
    {
        doMinify  |= (strcmp( args[i], "minify"  )==0);
    }

    bool res = true;

    // text input formats
    if( strcmp(typeStr,"Text_To_Text")==0 || strcmp(typeStr,"Text_To_CString")==0 || strcmp(typeStr,"Text_To_CArray")==0 )
    {
        size_t inputLen = 0;
        char *inputStr = utlReadAll(inputFileStr, &inputLen);
        if (!inputStr )
        {
           printf( "Error - could not read file \"%s\"...\n", inputFileStr);
           return -1;
        }

        if( doMinify )
        {
            if( !minify( inputStr ) )
            {
                printf( "Error - could not minify file \"%s\"...\n", outputFileStr);
                return -1;
            }
        }

        FILE *outputFile = fopen( outputFileStr, "wt" );
        if (!outputFile)
        {
            printf( "Error - could not create file \"%s\"...\n", outputFileStr);
            return -1;
        }

        if (strcmp(typeStr, "Text_To_Text") == 0)
        {
            res = iConvert_Text_To_Text(outputFile, inputStr, inputFileStr);
        }
        else if (strcmp(typeStr,"Text_To_CString")==0 )
        {
            res = iConvert_Text_To_CString(outputFile, inputStr, inputFileStr);
        }
        else // if (strcmp(typeStr,"Text_To_CArray")==0 )
        {
            res = iConvert_Text_To_CArray(outputFile, inputStr, inputFileStr);
        }

        fclose( outputFile );
        free(inputStr);
    }
    // binary output formats
    else if( strcmp(typeStr,"Binary_To_CArray")==0 )
    {
        res = iConvert_Bin_To_CArray(inputFileStr,outputFileStr);
    }
    // unkown format
    else
    {
        printf( "Error - Format \"%s\" is not known...\n", typeStr);
        return -1;
    }

    if( !res )
    {
        printf( "Error - could not convert \"%s\"...\n", typeStr);
        return -1;
    }


    printf( "toArray - \"%s\" converted correctly in format %s  %s\n", outputFileStr, typeStr, (doMinify)?"(minified)":"" );

    return 0;
}
